@@ -25,6 +25,34 @@
 
 static int s_in = -1, s_out = -1;
 
+/* ------------------------------------------------------------------
+ * sanitize_char:
+ *      helper function, returns only characters from isalnum+".:-"
+ *      the rest is replaced with underscore
+ *      note that is it better not to use isalnum() since it
+ *          is locale-sensitive
+ *
+ *      to satinize DNS records
+ * ------------------------------------------------------------------
+*/
+
+unsigned char sanitize_char(unsigned char c)
+{
+    unsigned short int i;
+    unsigned char r = '_';
+    i = (unsigned short int) c;
+    /* since we want to be 8-bit clean, we do not sanitize chars over 128,
+     * this would be OK for DNS, but not for ident response
+     */
+    if (i=='\0') /* so that we do not accidentally delete the end of string marker */
+        return c;
+    if (i>=32) {
+        if ( ('A'<=i && i<='Z') || ('a'<=i && i<='z') || ('0'<=i && i<='9') || (i=='.') || (i==':') || (i=='-') || (i>=128) )
+            r = c;
+    }
+    return r;
+}
+
 /* ------------------------------------------------------------------
  * lookup_addr:
  *      if resolve_addr, try to reverse resolve the address.
@@ -33,8 +61,9 @@ static int s_in = -1, s_out = -1;
  */
 static char *lookup_addr(struct in_addr in)
 {
-    static char addr[100];
+    static char addr[256];
     struct hostent *he;
+    unsigned short int i;
 
     if (resolve_addr) {
 	he = gethostbyaddr((char *) &in, sizeof(struct in_addr), AF_INET);
@@ -46,6 +75,11 @@ static char *lookup_addr(struct in_addr in)
 	strncpy(addr, inet_ntoa(in), sizeof(addr));
 
     addr[sizeof(addr)-1] = '\0';
+    for (i=0; i<sizeof(addr); i++) {
+        if (addr[i]=='\0')
+            break;
+        addr[i] = sanitize_char(addr[i]);
+    }
     return addr;
 }
 
@@ -148,6 +182,7 @@ void inetd_service(int sd_in, int sd_out)
     char buffer[MAX_SOCK_LENGTH];
     int sinsize = sizeof(struct sockaddr_in);
     int reqstat;
+    unsigned int i;
     char *identity = NULL;
     char *remote_address;
     char *nullident = "(null)";
@@ -170,10 +205,27 @@ void inetd_service(int sd_in, int sd_out)
     laddr = sin.sin_addr;
 
     reqstat = get_request(sd_in, buffer, MAX_SOCK_LENGTH);
+    buffer[sizeof(buffer)-1] = '\0';
+    for (i=0; i<sizeof(buffer); i++) {
+        if (buffer[i]=='\0')
+            break;
+        buffer[i] = sanitize_char(buffer[i]);
+    }
+
+
     if (use_ident)
         identity = ident_id(sd_in, IDENT_TIME);
     if (identity == NULL)
         identity = nullident;
+    else {
+        for (i=0; ; i++) {
+            if (identity[i]=='\0')
+                break;
+            identity[i] = sanitize_char(identity[i]);
+        }
+    }
+
+    /* remote_address is already sanitized in lookup_addr */
     remote_address = lookup_addr(raddr);
     do_finger(buffer, identity, remote_address, sd_out);
 
