@@ -40,11 +40,11 @@
  * ==================================================================
  */
 	/* SERVICE BEHAVIOR     */
-unsigned char   resolve_addr;		/* reverse lookup addresses	*/
-unsigned char   ignore_user;		/* ignore users' .efingerd file	*/
-unsigned char   use_ident;		/* use ident protocol	*/
-unsigned short	client_timeout;		/* number of seconds till disconnect */
-unsigned char	display_full_name;	/* display users' full name */
+extern unsigned char   resolve_addr;	/* reverse lookup addresses	*/
+extern unsigned char ignore_user;	/* ignore users' .efingerd file	*/
+extern unsigned char   use_ident;	/* use ident protocol	*/
+extern unsigned short	client_timeout;	/* number of seconds till disconnect */
+extern unsigned char display_full_name;	/* display users' full name */
 /* ==================================================================
  * PROTOTYPES:
  * ==================================================================
